@@ -114,7 +114,7 @@ typedef void (*ast_event_cb_t)(const struct ast_event *event, void *userdata);
  * \note A NULL description will cause this function to crash, so watch out!
  */
 struct ast_event_sub *ast_event_subscribe(enum ast_event_type event_type,
-       ast_event_cb_t cb, char *description, void *userdata, ...);
+       ast_event_cb_t cb, const char *description, void *userdata, ...);
 
 /*!
  * \brief Allocate a subscription, but do not activate it
@@ -397,8 +397,7 @@ int ast_event_queue(struct ast_event *event);
  * The event API already knows which events can be cached and how to cache them.
  *
  * \retval 0 success
- * \retval non-zero failure.  If failure is returned, the event must be destroyed
- *         by the caller of this function.
+ * \retval non-zero failure.
  */
 int ast_event_queue_and_cache(struct ast_event *event);
 
