@@ -23,14 +23,29 @@
 #ifndef _ASTERISK_STRINGS_H
 #define _ASTERISK_STRINGS_H
 
+/* #define DEBUG_OPAQUE */
+
 #include <ctype.h>
 
-#include "asterisk/inline_api.h"
 #include "asterisk/utils.h"
 #include "asterisk/threadstorage.h"
 
+#if defined(DEBUG_OPAQUE)
+#define __AST_STR_USED used2
+#define __AST_STR_LEN len2
+#define __AST_STR_STR str2
+#define __AST_STR_TS ts2
+#else
+#define __AST_STR_USED used
+#define __AST_STR_LEN len
+#define __AST_STR_STR str
+#define __AST_STR_TS ts
+#endif
+
 /* You may see casts in this header that may seem useless but they ensure this file is C++ clean */
 
+#define AS_OR(a,b)	(a && ast_str_strlen(a)) ? ast_str_buffer(a) : (b)
+
 #ifdef AST_DEVMODE
 #define ast_strlen_zero(foo)	_ast_strlen_zero(foo, __FILE__, __PRETTY_FUNCTION__, __LINE__)
 static force_inline int _ast_strlen_zero(const char *s, const char *file, const char *function, int line)
@@ -45,42 +60,45 @@ static force_inline int _ast_strlen_zero(const char *s, const char *file, const
 }
 
 #else
-static force_inline int ast_strlen_zero(const char *s)
+static force_inline int attribute_pure ast_strlen_zero(const char *s)
 {
 	return (!s || (*s == '\0'));
 }
 #endif
 
+#ifdef SENSE_OF_HUMOR
+#define ast_strlen_real(a)	(a) ? strlen(a) : 0
+#define ast_strlen_imaginary(a)	ast_random()
+#endif
+
 /*! \brief returns the equivalent of logic or for strings:
  * first one if not empty, otherwise second one.
  */
-#define S_OR(a, b)           (!ast_strlen_zero(a) ? (a) : (b))
+#define S_OR(a, b) ({typeof(&((a)[0])) __x = (a); ast_strlen_zero(__x) ? (b) : __x;})
 
 /*! \brief returns the equivalent of logic or for strings, with an additional boolean check:
  * second one if not empty and first one is true, otherwise third one.
  * example: S_COR(usewidget, widget, "<no widget>")
  */
-#define S_COR(a, b, c)   ((a && !ast_strlen_zero(b)) ? (b) : (c))
+#define S_COR(a, b, c) ({typeof(&((b)[0])) __x = (b); (a) && !ast_strlen_zero(__x) ? (__x) : (c);})
 
 /*!
   \brief Gets a pointer to the first non-whitespace character in a string.
-  \param ast_skip_blanks function being used
-  \arg str the input string
+  \param str the input string
   \return a pointer to the first non-whitespace character
  */
 AST_INLINE_API(
-char *ast_skip_blanks(const char *str),
+char * attribute_pure ast_skip_blanks(const char *str),
 {
 	while (*str && ((unsigned char) *str) < 33)
 		str++;
-	return (char *)str;
+	return (char *) str;
 }
 )
 
 /*!
   \brief Trims trailing whitespace characters from a string.
-  \param ast_skip_blanks function being used
-  \arg str the input string
+  \param str the input string
   \return a pointer to the modified string
  */
 AST_INLINE_API(
@@ -105,23 +123,21 @@ char *ast_trim_blanks(char *str),
 
 /*!
   \brief Gets a pointer to first whitespace character in a string.
-  \param ast_skip_noblanks function being used
-  \arg str the input string
+  \param str the input string
   \return a pointer to the first whitespace character
  */
 AST_INLINE_API(
-char *ast_skip_nonblanks(char *str),
+char * attribute_pure ast_skip_nonblanks(const char *str),
 {
 	while (*str && ((unsigned char) *str) > 32)
 		str++;
-	return str;
+	return (char *) str;
 }
 )
   
 /*!
   \brief Strip leading/trailing whitespace from a string.
-  \param ast_strip function ast_strip being used.
-  \arg s The string to be stripped (will be modified).
+  \param s The string to be stripped (will be modified).
   \return The stripped string.
 
   This functions strips all leading and trailing whitespace
@@ -131,9 +147,9 @@ char *ast_skip_nonblanks(char *str),
 AST_INLINE_API(
 char *ast_strip(char *s),
 {
-	s = ast_skip_blanks(s);
-	if (s)
+	if ((s = ast_skip_blanks(s))) {
 		ast_trim_blanks(s);
+	}
 	return s;
 } 
 )
@@ -180,9 +196,9 @@ char *ast_unescape_c(char *s);
 
 /*!
   \brief Size-limited null-terminating string copy.
-  \arg dst The destination buffer.
-  \arg src The source string
-  \arg size The size of the destination buffer
+  \param dst The destination buffer.
+  \param src The source string
+  \param size The size of the destination buffer
   \return Nothing.
 
   This is similar to \a strncpy, with two important differences:
@@ -206,7 +222,6 @@ void ast_copy_string(char *dst, const char *src, size_t size),
 }
 )
 
-
 /*!
   \brief Build a string in a buffer, designed to be called repeatedly
   
@@ -215,9 +230,9 @@ void ast_copy_string(char *dst, const char *src, size_t size),
   This is a wrapper for snprintf, that properly handles the buffer pointer
   and buffer space available.
 
-  \arg buffer current position in buffer to place string into (will be updated on return)
-  \arg space remaining space in buffer (will be updated on return)
-  \arg fmt printf-style format string
+  \param buffer current position in buffer to place string into (will be updated on return)
+  \param space remaining space in buffer (will be updated on return)
+  \param fmt printf-style format string
   \retval 0 on success
   \retval non-zero on failure.
 */
@@ -247,7 +262,7 @@ int ast_build_string_va(char **buffer, size_t *space, const char *fmt, va_list a
  * \retval -1 if "true".
  * \retval 0 otherwise.
  */
-int ast_true(const char *val);
+int attribute_pure ast_true(const char *val);
 
 /*! 
  * \brief Make sure something is false.
@@ -259,7 +274,7 @@ int ast_true(const char *val);
  * \retval -1 if "true".
  * \retval 0 otherwise.
  */
-int ast_false(const char *val);
+int attribute_pure ast_false(const char *val);
 
 /*
  *  \brief Join an array of strings into a single string.
@@ -271,7 +286,7 @@ int ast_false(const char *val);
  * string.  It will also place a space in the result buffer in between each
  * string from 'w'.
 */
-void ast_join(char *s, size_t len, char * const w[]);
+void ast_join(char *s, size_t len, const char * const w[]);
 
 /*
   \brief Parse a time (integer) string.
@@ -327,11 +342,16 @@ int ast_get_timeval(const char *src, struct timeval *tv, struct timeval _default
  *	ast_str_set_va(&buf, max_len, ap)
  *	ast_str_append_va(&buf, max_len, ap)
  *
- * \arg max_len The maximum allowed length, reallocating if needed.
+ * \param max_len The maximum allowed capacity of the ast_str. Note that
+ *  if the value of max_len is less than the current capacity of the
+ *  ast_str (as returned by ast_str_size), then the parameter is effectively
+ *  ignored.
  * 	0 means unlimited, -1 means "at most the available space"
  *
  * \return All the functions return <0 in case of error, or the
- *	length of the string added to the buffer otherwise.
+ *	length of the string added to the buffer otherwise. Note that
+ *	in most cases where an error is returned, characters ARE written
+ *	to the ast_str.
  */
 
 /*! \brief The descriptor of a dynamic string
@@ -342,24 +362,19 @@ int ast_get_timeval(const char *src, struct timeval *tv, struct timeval _default
  * struct ast_threadstorage pointer.
  */
 struct ast_str {
-	size_t len;	/*!< The current maximum length of the string */
-	size_t used;	/*!< Amount of space used */
-	struct ast_threadstorage *ts;	/*!< What kind of storage is this ? */
+	size_t __AST_STR_LEN;			/*!< The current maximum length of the string */
+	size_t __AST_STR_USED;			/*!< Amount of space used */
+	struct ast_threadstorage *__AST_STR_TS;	/*!< What kind of storage is this ? */
 #define DS_MALLOC	((struct ast_threadstorage *)1)
 #define DS_ALLOCA	((struct ast_threadstorage *)2)
 #define DS_STATIC	((struct ast_threadstorage *)3)	/* not supported yet */
-	char str[0];	/*!< The string buffer */
+	char __AST_STR_STR[0];			/*!< The string buffer */
 };
 
-#define ast_str_size(a)	((a)->len)
-#define	ast_str_strlen(a)	((a)->used)
-#define	ast_str_buffer(a)	((a)->str)
-#define	ast_str_update(a)	(a)->used = strlen((a)->str)
-
 /*!
  * \brief Create a malloc'ed dynamic length string
  *
- * \arg init_len This is the initial length of the string buffer
+ * \param init_len This is the initial length of the string buffer
  *
  * \return This function returns a pointer to the dynamic string length.  The
  *         result will be NULL in the case of a memory allocation error.
@@ -367,6 +382,26 @@ struct ast_str {
  * \note The result of this function is dynamically allocated memory, and must
  *       be free()'d after it is no longer needed.
  */
+#if (defined(MALLOC_DEBUG) && !defined(STANDALONE))
+#define	ast_str_create(a)	_ast_str_create(a,__FILE__,__LINE__,__PRETTY_FUNCTION__)
+AST_INLINE_API(
+struct ast_str * attribute_malloc _ast_str_create(size_t init_len,
+		const char *file, int lineno, const char *func),
+{
+	struct ast_str *buf;
+
+	buf = (struct ast_str *)__ast_calloc(1, sizeof(*buf) + init_len, file, lineno, func);
+	if (buf == NULL)
+		return NULL;
+
+	buf->__AST_STR_LEN = init_len;
+	buf->__AST_STR_USED = 0;
+	buf->__AST_STR_TS = DS_MALLOC;
+
+	return buf;
+}
+)
+#else
 AST_INLINE_API(
 struct ast_str * attribute_malloc ast_str_create(size_t init_len),
 {
@@ -375,14 +410,15 @@ struct ast_str * attribute_malloc ast_str_create(size_t init_len),
 	buf = (struct ast_str *)ast_calloc(1, sizeof(*buf) + init_len);
 	if (buf == NULL)
 		return NULL;
-	
-	buf->len = init_len;
-	buf->used = 0;
-	buf->ts = DS_MALLOC;
+
+	buf->__AST_STR_LEN = init_len;
+	buf->__AST_STR_USED = 0;
+	buf->__AST_STR_TS = DS_MALLOC;
 
 	return buf;
 }
 )
+#endif
 
 /*! \brief Reset the content of a dynamic string.
  * Useful before a series of ast_str_append.
@@ -391,13 +427,93 @@ AST_INLINE_API(
 void ast_str_reset(struct ast_str *buf),
 {
 	if (buf) {
-		buf->used = 0;
-		if (buf->len)
-			buf->str[0] = '\0';
+		buf->__AST_STR_USED = 0;
+		if (buf->__AST_STR_LEN) {
+			buf->__AST_STR_STR[0] = '\0';
+		}
 	}
 }
 )
 
+/*! \brief Update the length of the buffer, after using ast_str merely as a buffer.
+ *  \param buf A pointer to the ast_str string.
+ */
+AST_INLINE_API(
+void ast_str_update(struct ast_str *buf),
+{
+	buf->__AST_STR_USED = strlen(buf->__AST_STR_STR);
+}
+)
+
+/*! \brief Trims trailing whitespace characters from an ast_str string.
+ *  \param buf A pointer to the ast_str string.
+ */
+AST_INLINE_API(
+void ast_str_trim_blanks(struct ast_str *buf),
+{
+	if (!buf) {
+		return;
+	}
+	while (buf->__AST_STR_USED && buf->__AST_STR_STR[buf->__AST_STR_USED - 1] < 33) {
+		buf->__AST_STR_STR[--(buf->__AST_STR_USED)] = '\0';
+	}
+}
+)
+
+/*!\brief Returns the current length of the string stored within buf.
+ * \param buf A pointer to the ast_str structure.
+ */
+AST_INLINE_API(
+size_t attribute_pure ast_str_strlen(const struct ast_str *buf),
+{
+	return buf->__AST_STR_USED;
+}
+)
+
+/*!\brief Returns the current maximum length (without reallocation) of the current buffer.
+ * \param buf A pointer to the ast_str structure.
+ * \retval Current maximum length of the buffer.
+ */
+AST_INLINE_API(
+size_t attribute_pure ast_str_size(const struct ast_str *buf),
+{
+	return buf->__AST_STR_LEN;
+}
+)
+
+/*!\brief Returns the string buffer within the ast_str buf.
+ * \param buf A pointer to the ast_str structure.
+ * \retval A pointer to the enclosed string.
+ */
+AST_INLINE_API(
+char * attribute_pure ast_str_buffer(const struct ast_str *buf),
+{
+	/* for now, cast away the const qualifier on the pointer
+	 * being returned; eventually, it should become truly const
+	 * and only be modified via accessor functions
+	 */
+	return (char *) buf->__AST_STR_STR;
+}
+)
+
+/*!\brief Truncates the enclosed string to the given length.
+ * \param buf A pointer to the ast_str structure.
+ * \param len Maximum length of the string.
+ * \retval A pointer to the resulting string.
+ */
+AST_INLINE_API(
+char *ast_str_truncate(struct ast_str *buf, ssize_t len),
+{
+	if (len < 0) {
+		buf->__AST_STR_USED += ((ssize_t) abs(len)) > (ssize_t) buf->__AST_STR_USED ? -buf->__AST_STR_USED : len;
+	} else {
+		buf->__AST_STR_USED = len;
+	}
+	buf->__AST_STR_STR[buf->__AST_STR_USED] = '\0';
+	return buf->__AST_STR_STR;
+}
+)
+	
 /*
  * AST_INLINE_API() is a macro that takes a block of code as an argument.
  * Using preprocessor #directives in the argument is not supported by all
@@ -415,47 +531,93 @@ void ast_str_reset(struct ast_str *buf),
 /*!
  * Make space in a new string (e.g. to read in data from a file)
  */
+#if (defined(MALLOC_DEBUG) && !defined(STANDALONE))
 AST_INLINE_API(
-int ast_str_make_space(struct ast_str **buf, size_t new_len),
+int _ast_str_make_space(struct ast_str **buf, size_t new_len, const char *file, int lineno, const char *function),
 {
-	_DB1(struct ast_str *old_buf = *buf;)
+	struct ast_str *old_buf = *buf;
 
-	if (new_len <= (*buf)->len) 
+	if (new_len <= (*buf)->__AST_STR_LEN) 
 		return 0;	/* success */
-	if ((*buf)->ts == DS_ALLOCA || (*buf)->ts == DS_STATIC)
+	if ((*buf)->__AST_STR_TS == DS_ALLOCA || (*buf)->__AST_STR_TS == DS_STATIC)
 		return -1;	/* cannot extend */
-	*buf = (struct ast_str *)ast_realloc(*buf, new_len + sizeof(struct ast_str));
-	if (*buf == NULL) /* XXX watch out, we leak memory here */
+	*buf = (struct ast_str *)__ast_realloc(*buf, new_len + sizeof(struct ast_str), file, lineno, function);
+	if (*buf == NULL) {
+		*buf = old_buf;
 		return -1;
-	if ((*buf)->ts != DS_MALLOC) {
-		pthread_setspecific((*buf)->ts->key, *buf);
+	}
+	if ((*buf)->__AST_STR_TS != DS_MALLOC) {
+		pthread_setspecific((*buf)->__AST_STR_TS->key, *buf);
 		_DB1(__ast_threadstorage_object_replace(old_buf, *buf, new_len + sizeof(struct ast_str));)
 	}
 
-        (*buf)->len = new_len;
-        return 0;
+	(*buf)->__AST_STR_LEN = new_len;
+	return 0;
+}
+)
+#define ast_str_make_space(a,b)	_ast_str_make_space(a,b,__FILE__,__LINE__,__PRETTY_FUNCTION__)
+#else
+AST_INLINE_API(
+int ast_str_make_space(struct ast_str **buf, size_t new_len),
+{
+	struct ast_str *old_buf = *buf;
+
+	if (new_len <= (*buf)->__AST_STR_LEN) 
+		return 0;	/* success */
+	if ((*buf)->__AST_STR_TS == DS_ALLOCA || (*buf)->__AST_STR_TS == DS_STATIC)
+		return -1;	/* cannot extend */
+	*buf = (struct ast_str *)ast_realloc(*buf, new_len + sizeof(struct ast_str));
+	if (*buf == NULL) {
+		*buf = old_buf;
+		return -1;
+	}
+	if ((*buf)->__AST_STR_TS != DS_MALLOC) {
+		pthread_setspecific((*buf)->__AST_STR_TS->key, *buf);
+		_DB1(__ast_threadstorage_object_replace(old_buf, *buf, new_len + sizeof(struct ast_str));)
+	}
+
+	(*buf)->__AST_STR_LEN = new_len;
+	return 0;
+}
+)
+#endif
+
+AST_INLINE_API(
+int ast_str_copy_string(struct ast_str **dst, struct ast_str *src),
+{
+
+	/* make sure our destination is large enough */
+	if (src->__AST_STR_USED + 1 > (*dst)->__AST_STR_LEN) {
+		if (ast_str_make_space(dst, src->__AST_STR_USED + 1)) {
+			return -1;
+		}
+	}
+
+	memcpy((*dst)->__AST_STR_STR, src->__AST_STR_STR, src->__AST_STR_USED + 1);
+	(*dst)->__AST_STR_USED = src->__AST_STR_USED;
+	return 0;
 }
 )
 
 #define ast_str_alloca(init_len)			\
 	({						\
-		struct ast_str *buf;			\
-		buf = alloca(sizeof(*buf) + init_len);	\
-		buf->len = init_len;			\
-		buf->used = 0;				\
-		buf->ts = DS_ALLOCA;			\
-		buf->str[0] = '\0';			\
-		(buf);					\
+		struct ast_str *__ast_str_buf;			\
+		__ast_str_buf = alloca(sizeof(*__ast_str_buf) + init_len);	\
+		__ast_str_buf->__AST_STR_LEN = init_len;			\
+		__ast_str_buf->__AST_STR_USED = 0;				\
+		__ast_str_buf->__AST_STR_TS = DS_ALLOCA;			\
+		__ast_str_buf->__AST_STR_STR[0] = '\0';			\
+		(__ast_str_buf);					\
 	})
 
 /*!
  * \brief Retrieve a thread locally stored dynamic string
  *
- * \arg ts This is a pointer to the thread storage structure declared by using
+ * \param ts This is a pointer to the thread storage structure declared by using
  *      the AST_THREADSTORAGE macro.  If declared with 
  *      AST_THREADSTORAGE(my_buf, my_buf_init), then this argument would be 
  *      (&my_buf).
- * \arg init_len This is the initial length of the thread's dynamic string. The
+ * \param init_len This is the initial length of the thread's dynamic string. The
  *      current length may be bigger if previous operations in this thread have
  *      caused it to increase.
  *
@@ -489,11 +651,11 @@ struct ast_str *ast_str_thread_get(struct ast_threadstorage *ts,
 	buf = (struct ast_str *)ast_threadstorage_get(ts, sizeof(*buf) + init_len);
 	if (buf == NULL)
 		return NULL;
-	
-	if (!buf->len) {
-		buf->len = init_len;
-		buf->used = 0;
-		buf->ts = ts;
+
+	if (!buf->__AST_STR_LEN) {
+		buf->__AST_STR_LEN = init_len;
+		buf->__AST_STR_USED = 0;
+		buf->__AST_STR_TS = ts;
 	}
 
 	return buf;
@@ -509,11 +671,11 @@ struct ast_str *__ast_str_thread_get(struct ast_threadstorage *ts,
 	buf = (struct ast_str *)__ast_threadstorage_get(ts, sizeof(*buf) + init_len, file, function, line);
 	if (buf == NULL)
 		return NULL;
-	
-	if (!buf->len) {
-		buf->len = init_len;
-		buf->used = 0;
-		buf->ts = ts;
+
+	if (!buf->__AST_STR_LEN) {
+		buf->__AST_STR_LEN = init_len;
+		buf->__AST_STR_USED = 0;
+		buf->__AST_STR_TS = ts;
 	}
 
 	return buf;
@@ -526,36 +688,65 @@ struct ast_str *__ast_str_thread_get(struct ast_threadstorage *ts,
 /*!
  * \brief Error codes from __ast_str_helper()
  * The undelying processing to manipulate dynamic string is done
- * by __ast_str_helper(), which can return a success, a
- * permanent failure (e.g. no memory), or a temporary one (when
- * the string needs to be reallocated, and we must run va_start()
- * again; XXX this convoluted interface is only here because
- * FreeBSD 4 lacks va_copy, but this will be fixed and the
- * interface simplified).
+ * by __ast_str_helper(), which can return a success or a
+ * permanent failure (e.g. no memory).
  */
 enum {
-	/*! An error has occured and the contents of the dynamic string
+	/*! An error has occurred and the contents of the dynamic string
 	 *  are undefined */
 	AST_DYNSTR_BUILD_FAILED = -1,
 	/*! The buffer size for the dynamic string had to be increased, and
 	 *  __ast_str_helper() needs to be called again after
-	 *  a va_end() and va_start().
+	 *  a va_end() and va_start().  This return value is legacy and will
+	 *  no longer be used.
 	 */
 	AST_DYNSTR_BUILD_RETRY = -2
 };
 
+/*!
+ * \brief Core functionality of ast_str_(set|append)_va
+ *
+ * The arguments to this function are the same as those described for
+ * ast_str_set_va except for an addition argument, append.
+ * If append is non-zero, this will append to the current string instead of
+ * writing over it.
+ *
+ * AST_DYNSTR_BUILD_RETRY is a legacy define.  It should probably never
+ * again be used.
+ *
+ * A return of AST_DYNSTR_BUILD_FAILED indicates a memory allocation error.
+ *
+ * A return value greater than or equal to zero indicates the number of
+ * characters that have been written, not including the terminating '\0'.
+ * In the append case, this only includes the number of characters appended.
+ *
+ * \note This function should never need to be called directly.  It should
+ *       through calling one of the other functions or macros defined in this
+ *       file.
+ */
+#if (defined(MALLOC_DEBUG) && !defined(STANDALONE))
+int __attribute__((format(printf, 4, 0))) __ast_debug_str_helper(struct ast_str **buf, ssize_t max_len,
+							   int append, const char *fmt, va_list ap, const char *file, int lineno, const char *func);
+#define __ast_str_helper(a,b,c,d,e)	__ast_debug_str_helper(a,b,c,d,e,__FILE__,__LINE__,__PRETTY_FUNCTION__)
+#else
+int __attribute__((format(printf, 4, 0))) __ast_str_helper(struct ast_str **buf, ssize_t max_len,
+							   int append, const char *fmt, va_list ap);
+#endif
+char *__ast_str_helper2(struct ast_str **buf, ssize_t max_len,
+	const char *src, size_t maxsrc, int append, int escapecommas);
+
 /*!
  * \brief Set a dynamic string from a va_list
  *
- * \arg buf This is the address of a pointer to a struct ast_str.
+ * \param buf This is the address of a pointer to a struct ast_str.
  *	If it is retrieved using ast_str_thread_get, the
 	struct ast_threadstorage pointer will need to
  *      be updated in the case that the buffer has to be reallocated to
  *      accommodate a longer string than what it currently has space for.
- * \arg max_len This is the maximum length to allow the string buffer to grow
+ * \param max_len This is the maximum length to allow the string buffer to grow
  *      to.  If this is set to 0, then there is no maximum length.
- * \arg fmt This is the format string (printf style)
- * \arg ap This is the va_list
+ * \param fmt This is the format string (printf style)
+ * \param ap This is the va_list
  *
  * \return The return value of this function is the same as that of the printf
  *         family of functions.
@@ -581,74 +772,63 @@ enum {
  *      ...
  * }
  * \endcode
- *
- * \note: the following two functions must be implemented as macros
- *	because we must do va_end()/va_start() on the original arguments.
  */
-#define ast_str_set_va(buf, max_len, fmt, ap)			\
-	({								\
-		int __res;						\
-		while ((__res = __ast_str_helper(buf, max_len,		\
-			0, fmt, ap)) == AST_DYNSTR_BUILD_RETRY) {	\
-			va_end(ap);					\
-			va_start(ap, fmt);				\
-		}							\
-		(__res);						\
-	})
+AST_INLINE_API(int __attribute__((format(printf, 3, 0))) ast_str_set_va(struct ast_str **buf, ssize_t max_len, const char *fmt, va_list ap),
+{
+	return __ast_str_helper(buf, max_len, 0, fmt, ap);
+}
+)
 
 /*!
  * \brief Append to a dynamic string using a va_list
  *
  * Same as ast_str_set_va(), but append to the current content.
  */
-#define ast_str_append_va(buf, max_len, fmt, ap)		\
-	({								\
-		int __res;						\
-		while ((__res = __ast_str_helper(buf, max_len,		\
-			1, fmt, ap)) == AST_DYNSTR_BUILD_RETRY) {	\
-			va_end(ap);					\
-			va_start(ap, fmt);				\
-		}							\
-		(__res);						\
-	})
+AST_INLINE_API(int __attribute__((format(printf, 3, 0))) ast_str_append_va(struct ast_str **buf, ssize_t max_len, const char *fmt, va_list ap),
+{
+	return __ast_str_helper(buf, max_len, 1, fmt, ap);
+}
+)
 
-/*!
- * \brief Core functionality of ast_str_(set|append)_va
- *
- * The arguments to this function are the same as those described for
- * ast_str_set_va except for an addition argument, append.
- * If append is non-zero, this will append to the current string instead of
- * writing over it.
- *
- * In the case that this function is called and the buffer was not large enough
- * to hold the result, the partial write will be truncated, and the result
- * AST_DYNSTR_BUILD_RETRY will be returned to indicate that the buffer size
- * was increased, and the function should be called a second time.
- *
- * A return of AST_DYNSTR_BUILD_FAILED indicates a memory allocation error.
- *
- * A return value greater than or equal to zero indicates the number of
- * characters that have been written, not including the terminating '\0'.
- * In the append case, this only includes the number of characters appended.
- *
- * \note This function should never need to be called directly.  It should
- *       through calling one of the other functions or macros defined in this
- *       file.
- */
-int __attribute__((format(printf, 4, 0))) __ast_str_helper(struct ast_str **buf, size_t max_len,
-							   int append, const char *fmt, va_list ap);
+/*!\brief Set a dynamic string to a non-NULL terminated substring. */
+AST_INLINE_API(char *ast_str_set_substr(struct ast_str **buf, ssize_t maxlen, const char *src, size_t maxsrc),
+{
+	return __ast_str_helper2(buf, maxlen, src, maxsrc, 0, 0);
+}
+)
+
+/*!\brief Append a non-NULL terminated substring to the end of a dynamic string. */
+AST_INLINE_API(char *ast_str_append_substr(struct ast_str **buf, ssize_t maxlen, const char *src, size_t maxsrc),
+{
+	return __ast_str_helper2(buf, maxlen, src, maxsrc, 1, 0);
+}
+)
+
+/*!\brief Set a dynamic string to a non-NULL terminated substring, with escaping of commas. */
+AST_INLINE_API(char *ast_str_set_escapecommas(struct ast_str **buf, ssize_t maxlen, const char *src, size_t maxsrc),
+{
+	return __ast_str_helper2(buf, maxlen, src, maxsrc, 0, 1);
+}
+)
+
+/*!\brief Append a non-NULL terminated substring to the end of a dynamic string, with escaping of commas. */
+AST_INLINE_API(char *ast_str_append_escapecommas(struct ast_str **buf, ssize_t maxlen, const char *src, size_t maxsrc),
+{
+	return __ast_str_helper2(buf, maxlen, src, maxsrc, 1, 1);
+}
+)
 
 /*!
  * \brief Set a dynamic string using variable arguments
  *
- * \arg buf This is the address of a pointer to a struct ast_str which should
+ * \param buf This is the address of a pointer to a struct ast_str which should
  *      have been retrieved using ast_str_thread_get.  It will need to
  *      be updated in the case that the buffer has to be reallocated to
  *      accomodate a longer string than what it currently has space for.
- * \arg max_len This is the maximum length to allow the string buffer to grow
+ * \param max_len This is the maximum length to allow the string buffer to grow
  *      to.  If this is set to 0, then there is no maximum length.
  *	If set to -1, we are bound to the current maximum length.
- * \arg fmt This is the format string (printf style)
+ * \param fmt This is the format string (printf style)
  *
  * \return The return value of this function is the same as that of the printf
  *         family of functions.
@@ -657,7 +837,7 @@ int __attribute__((format(printf, 4, 0))) __ast_str_helper(struct ast_str **buf,
  */
 AST_INLINE_API(
 int __attribute__((format(printf, 3, 4))) ast_str_set(
-	struct ast_str **buf, size_t max_len, const char *fmt, ...),
+	struct ast_str **buf, ssize_t max_len, const char *fmt, ...),
 {
 	int res;
 	va_list ap;
@@ -678,7 +858,7 @@ int __attribute__((format(printf, 3, 4))) ast_str_set(
  */
 AST_INLINE_API(
 int __attribute__((format(printf, 3, 4))) ast_str_append(
-	struct ast_str **buf, size_t max_len, const char *fmt, ...),
+	struct ast_str **buf, ssize_t max_len, const char *fmt, ...),
 {
 	int res;
 	va_list ap;
@@ -699,7 +879,7 @@ int __attribute__((format(printf, 3, 4))) ast_str_append(
  *
  * http://www.cse.yorku.ca/~oz/hash.html
  */
-static force_inline int ast_str_hash(const char *str)
+static force_inline int attribute_pure ast_str_hash(const char *str)
 {
 	int hash = 5381;
 
@@ -709,6 +889,29 @@ static force_inline int ast_str_hash(const char *str)
 	return abs(hash);
 }
 
+/*!
+ * \brief Compute a hash value on a string
+ *
+ * \param[in] str The string to add to the hash
+ * \param[in] hash The hash value to add to
+ * 
+ * \details
+ * This version of the function is for when you need to compute a
+ * string hash of more than one string.
+ *
+ * This famous hash algorithm was written by Dan Bernstein and is
+ * commonly used.
+ *
+ * \sa http://www.cse.yorku.ca/~oz/hash.html
+ */
+static force_inline int ast_str_hash_add(const char *str, int hash)
+{
+	while (*str)
+		hash = hash * 33 ^ *str++;
+
+	return abs(hash);
+}
+
 /*!
  * \brief Compute a hash value on a case-insensitive string
  *
@@ -716,7 +919,7 @@ static force_inline int ast_str_hash(const char *str)
  * all characters to lowercase prior to computing a hash. This
  * allows for easy case-insensitive lookups in a hash table.
  */
-static force_inline int ast_str_case_hash(const char *str)
+static force_inline int attribute_pure ast_str_case_hash(const char *str)
 {
 	int hash = 5381;
 
@@ -726,4 +929,5 @@ static force_inline int ast_str_case_hash(const char *str)
 
 	return abs(hash);
 }
+
 #endif /* _ASTERISK_STRINGS_H */
